import CSSClasses from '../Helpers/CSSClasses.js';
import Conditions from '../Helpers/Conditions.js';
import ValueCallback from '../Helpers/ValueCallback.js';

/**
 * Field class.
 * 
 * Should be extended by all field types and not used directly.
 * 
 * @since 1.0.0
 */
export default class Field {
    /**
     * Initializing field.
     * 
     * @param fieldData  - Field data.
     * @param {Array} formValues - Form values.
     * 
     * @since 1.0.0
     */
    constructor(fieldData, formValues) {
        this.formValues = formValues;

        this.id = fieldData.id;
        this.type = fieldData.type;
        this.label = fieldData.label;
        this.prefix = fieldData.prefix;
        this.suffix = fieldData.suffix;
        this.placeholder = fieldData.placeholder;
        this.options = fieldData.options;
        this.help = fieldData.help;

        this.valueDefault = fieldData.valueDefault;
        this.valueCallback = fieldData.valueCallback !== undefined ? new ValueCallback(fieldData.valueCallback, formValues) : undefined;

        this.validated = false;
        this.errors = [];
        this.validations = fieldData.validations
        this.replacements = fieldData.replacements;
        this.conditions = fieldData.conditions;

        this.labelClasses = new CSSClasses(fieldData.labelClasses)
        this.inputClasses = new CSSClasses(fieldData.inputClasses);
        this.classes = new CSSClasses(fieldData.classes);
        this.size = fieldData.size;
    }

    /**
     * Updating field assets
     * 
     * @since 1.0.0
     */
    updateAssets() {
        if (!this.validated) {
            return;
        }

        if (this.getErrors().length > 0) {
            this.classes.remove('is-valid');
            this.inputClasses.remove('is-valid');
            this.classes.add('is-invalid');
            this.inputClasses.add('is-invalid');
        } else {
            this.classes.remove('is-invalid');
            this.inputClasses.remove('is-invalid');
            this.classes.add('is-valid');
            this.inputClasses.add('is-valid');
        }
    }

    /**
     * Replace a value by field replacements.
     * 
     * @param {any} value - Value to be replaced.
     * 
     * @returns {any} Replaced value.
     * 
     * @since 1.0.0
     */
    replace(value) {
        if (this.replacements === undefined) {
            return value;
        }

        this.replacements.forEach(replacement => {
            value = value.replace(replacement.from, replacement.to);
        });

        return value;
    }

    /**
     * Checks if conditions for displaying field are met.
     * 
     * @returns True if conditions are met, false otherwise.
     * 
     * @since 1.0.0
     */
    conditionsFulfilled() {
        if (this.conditions === undefined || this.conditions.length === 0) {
            return true;
        }

        let conditions = new Conditions(this.conditions, this.formValues);
        return conditions.fullfilled();
    }

    /**
     * Get errors of the field.
     * 
     * @returns {Array} Erros of the field.
     * 
     * @since 1.0.0
     */
    getErrors() {
        return this.errors;
    }

    /**
     * Is field visible.
     * 
     * @returns {boolean} True if field is visible, false otherwise.
     * 
     * @since 1.0.0
     */
    isVisible() {
        return this.conditionsFulfilled();
    }

    /**
     * Is field completed.
     * 
     * @returns {boolean} True if field is completed, false otherwise.
     * 
     * @since 1.0.0
     */
    isComplete() {
        if (!this.conditionsFulfilled()) {
            return true;
        }

        if (this.isValidated() && this.getErrors().length === 0) {
            return true;
        }

        return false;
    }

    /**
     * Is field validated.
     * 
     * @returns {boolean} True if field is validated, false otherwise.
     * 
     * @since 1.0.0
     */
    isValidated() {
        return this.validated;
    }

    /**
     * Validate field.
     * 
     * @returns True if field is valid, false otherwise.
     * 
     * @since 1.0.0
     */
    validate(){}
}